<?php

namespace Nwb\CyberSecModule\Config;

/**
 * Configuration des patterns de détection de menaces
 *
 * Ce fichier centralise tous les motifs utilisés pour détecter
 * les tentatives de piratage et facilite leur maintenance.
 */
class SecurityPatterns
{
    /**
     * Patterns de détection pour les URI malveillantes
     *
     * @return array
     */
    public static function getUriPatterns(): array
    {
        return [
            // === PATTERNS WORDPRESS ===
            '/wp-login.php',
            '/wp-admin',
            '/wordpress',
            '/wp-',
            '/wp-content/plugins/init-help/init.php',

            // === FICHIERS SENSIBLES ===
            '/.env',
            '/.git',
            '.php.bak',
            '.htaccess',
            '.htpasswd',
            '.git/config',
            '/.well-known/security.txt',

            // === FICHIERS MALVEILLANTS SPÉCIFIQUES ===
            // Backdoors et shells web communs
            'alf.php',
            'file5.php',
            'Jcrop.php',
            'dropdown.php',
            'css.php',
            'autoload_classmap.php',
            'lv.php',
            'cloud.php',
            'phpinfo.php',
            '/test.php',
            '/info.php',
            '/debug.php',

            // === RÉPERTOIRES SENSIBLES ===
            '/cgi-bin/',
            '/plugins/',
            '/elfinder/',
            '/phpmyadmin',
            '/admin',
            '/custom-admin',
            '/secret-path',
            '/backup/',
            '/config/',

            // === FONCTIONS DANGEREUSES ===
            'eval(',
            'base64_decode',
            
            // === INJECTION SQL DANS URI ===
            'UNION SELECT',
            'UNION+SELECT',
            'union+select',
            'union%20select',
            'DROP TABLE',
            'DROP+TABLE',
            'drop+table',
            'drop%20table',
            'OR 1=1',
            'OR+1=1',
            'or+1=1',
            'or%201=1',
            'AND 1=1',
            'AND+1=1',
            'and+1=1',
            'and%201=1',
            'INSERT INTO',
            'insert+into',
            'insert%20into',
            'DELETE FROM',
            'delete+from',
            'delete%20from',
            'UPDATE SET',
            'update+set',
            'update%20set',
            'CONCAT(',
            'concat(',
            'CHAR(',
            'char(',
            'ASCII(',
            'ascii(',
            'SUBSTRING(',
            'substring(',
            'LOAD_FILE(',
            'load_file(',
            'INTO OUTFILE',
            'into+outfile',
            'into%20outfile',
            'INFORMATION_SCHEMA',
            'information_schema',
            'mysql.user',
            'pg_user',
            'sysobjects',
            'syscolumns',
            
            // === XSS DANS URI ===
            '<script',
            '<script>',
            '<script%20',
            '<iframe',
            '<iframe>',
            '<iframe%20',
            'javascript:',
            'javascript%3A',
            'onload=',
            'onload%3D',
            'onerror=',
            'onerror%3D',
            'onclick=',
            'onclick%3D',
            'onmouseover=',
            'onmouseover%3D',
            'onfocus=',
            'onfocus%3D',
            'onblur=',
            'onblur%3D',
            'onchange=',
            'onchange%3D',
            'onsubmit=',
            'onsubmit%3D',
            'document.cookie',
            'document%2Ecookie',
            'window.location',
            'window%2Elocation',
            'alert(',
            'alert%28',
            'confirm(',
            'confirm%28',
            'prompt(',
            'prompt%28',
            'String.fromCharCode',
            'string%2EfromCharCode',
            'eval(',
            'eval%28',
            
            // === PATH TRAVERSAL DANS URI ===
            '../',
            '..%2F',
            '..%252F',
            '..\\',
            '..%5C',
            '..%255C',
            '..../',
            '....%2F',
            '....\\',
            '....%5C',
            '%2e%2e%2f',
            '%2e%2e/',
            '%2e%2e%5c',
            '%2e%2e\\',
            '..%c0%af',
            '..%c1%9c',
            '.%2e/',
            '.%2e%2f',
            '.%2e\\',
            '.%2e%5c',
            
            // === INCLUSION DE FICHIERS DANS URI ===
            'php://input',
            'php%3A//input',
            'php://filter',
            'php%3A//filter',
            'php://wrapper',
            'php%3A//wrapper',
            'file://',
            'file%3A//',
            'http://',
            'https://',
            'ftp://',
            'ftps://',
            'data:',
            'data%3A',
            'expect://',
            'expect%3A//',
            
            // === COMMANDES SYSTÈME DANS URI ===
            'system(',
            'system%28',
            'exec(',
            'exec%28',
            'shell_exec(',
            'shell_exec%28',
            'passthru(',
            'passthru%28',
            'popen(',
            'popen%28',
            'proc_open(',
            'proc_open%28',
            
            // === ENCODAGE MALVEILLANT ===
            '%00',  // Null byte
            '%0a',  // Line feed
            '%0d',  // Carriage return
            '%09',  // Tab
            '%20and%20',
            '%20or%20',
            '%20union%20',
            '%27',  // Single quote
            '%22',  // Double quote
            '%3C',  // <
            '%3E',  // >
            '%28',  // (
            '%29',  // )
        ];
    }

    /**
     * Patterns de détection pour les données POST malveillantes
     *
     * @return array
     */
    public static function getPostPatterns(): array
    {
        return [
            // === INJECTION SQL ===
            'UNION\s+SELECT',
            'UNION SELECT',
            'UNION+SELECT',
            'union+select',
            'union%20select',
            'DROP\s+TABLE',
            'DROP TABLE',
            'DROP+TABLE',
            'drop+table',
            'drop%20table',
            'DROP\s+DATABASE',
            'TRUNCATE\s+TABLE',
            'INSERT\s+INTO',
            'INSERT INTO',
            'insert+into',
            'insert%20into',
            'DELETE\s+FROM',
            'DELETE FROM',
            'delete+from',
            'delete%20from',
            'UPDATE\s+SET',
            'UPDATE SET',
            'update+set',
            'update%20set',
            'OR 1=1',
            'OR+1=1',
            'or+1=1',
            'or%201=1',
            'AND 1=1',
            'AND+1=1',
            'and+1=1',
            'and%201=1',
            'CONCAT(',
            'concat(',
            'CHAR(',
            'char(',
            'ASCII(',
            'ascii(',
            'SUBSTRING(',
            'substring(',
            'LOAD_FILE(',
            'load_file(',
            'INTO OUTFILE',
            'into+outfile',
            'into%20outfile',
            'INFORMATION_SCHEMA',
            'information_schema',
            'mysql.user',
            'pg_user',
            'sysobjects',
            'syscolumns',
            'CUSTOM_INJECTION',

            // === XSS (Cross-Site Scripting) ===
            '<script',
            '<script>',
            '<script%20',
            '<iframe',
            '<iframe>',
            '<iframe%20',
            'javascript:',
            'javascript%3A',
            'onload=',
            'onload%3D',
            'onerror=',
            'onerror%3D',
            'onclick=',
            'onclick%3D',
            'onmouseover=',
            'onmouseover%3D',
            'onfocus=',
            'onfocus%3D',
            'onblur=',
            'onblur%3D',
            'onchange=',
            'onchange%3D',
            'onsubmit=',
            'onsubmit%3D',
            'document\.cookie',
            'document.cookie',
            'document%2Ecookie',
            'window\.location',
            'window.location',
            'window%2Elocation',
            'alert(',
            'alert%28',
            'confirm(',
            'confirm%28',
            'prompt(',
            'prompt%28',
            'String.fromCharCode',
            'string%2EfromCharCode',

            // === EXECUTION DE CODE ===
            'eval\(',
            'eval(',
            'eval%28',
            'base64_decode',
            'exec\(',
            'exec(',
            'exec%28',
            'system\(',
            'system(',
            'system%28',
            'shell_exec\(',
            'shell_exec(',
            'shell_exec%28',
            'passthru\(',
            'passthru(',
            'passthru%28',
            'popen(',
            'popen%28',
            'proc_open(',
            'proc_open%28',

            // === INCLUSION DE FICHIERS ===
            'include\(',
            'require\(',
            'file_get_contents\(',
            'php://input',
            'php%3A//input',
            'php://filter',
            'php%3A//filter',
            'php://wrapper',
            'php%3A//wrapper',
            'file://',
            'file%3A//',
            'data:',
            'data%3A',
            'expect://',
            'expect%3A//',

            // === PATH TRAVERSAL ===
            '../',
            '..%2F',
            '..%252F',
            '..\\',
            '..%5C',
            '..%255C',
            '..../',
            '....%2F',
            '....\\',
            '....%5C',
            '%2e%2e%2f',
            '%2e%2e/',
            '%2e%2e%5c',
            '%2e%2e\\',
            '..%c0%af',
            '..%c1%9c',
            '.%2e/',
            '.%2e%2f',
            '.%2e\\',
            '.%2e%5c',
            
            // === ENCODAGE MALVEILLANT ===
            '%00',  // Null byte
            '%0a',  // Line feed
            '%0d',  // Carriage return
            '%09',  // Tab
            '%20and%20',
            '%20or%20',
            '%20union%20',
            '%27',  // Single quote
            '%22',  // Double quote
            '%3C',  // <
            '%3E',  // >
            '%28',  // (
            '%29',  // )
        ];
    }

    /**
     * Patterns de détection pour les User-Agent malveillants
     *
     * @return array
     */
    public static function getHeaderPatterns(): array
    {
        return [
            // === OUTILS DE SCAN AUTOMATISÉS ===
            'sqlmap',
            'nikto',
            'nessus',
            'openvas',
            'nmap',
            'masscan',
            'custom-scanner',
            'penetration-test',
            'security-audit',
            'vulnerability-scanner',

            // === BOTS MALVEILLANTS ===
            'evil-bot',
            'bad-bot',
            'malicious',

            // === OUTILS D'EXPLOITATION ===
            'metasploit',
            'burpsuite',
            'havij',
            'acunetix',

            // === CRAWLERS AGRESSIFS ===
            'python-requests',
            'curl/',
            'wget/',

            // === SIGNATURES SPÉCIFIQUES ===
            'scanner',
            'exploit',
            'payload'
        ];
    }

    /**
     * Récupère tous les patterns organisés par catégorie
     *
     * @return array
     */
    public static function getAllPatterns(): array
    {
        return [
            'uri' => self::getUriPatterns(),
            'post' => self::getPostPatterns(),
            'headers' => self::getHeaderPatterns()
        ];
    }

    /**
     * Ajoute des patterns personnalisés à une catégorie existante
     *
     * @param string $category Catégorie (uri, post, headers)
     * @param array $patterns Nouveaux patterns à ajouter
     * @return array Patterns fusionnés
     */
    public static function addCustomPatterns(string $category, array $patterns): array
    {
        $allPatterns = self::getAllPatterns();

        if (!isset($allPatterns[$category])) {
            throw new \InvalidArgumentException("Catégorie invalide: $category");
        }

        $allPatterns[$category] = array_merge($allPatterns[$category], $patterns);

        return $allPatterns;
    }

    /**
     * Charge les patterns depuis un fichier JSON externe (optionnel)
     *
     * @param string $filePath Chemin vers le fichier JSON
     * @return array|null Patterns chargés ou null si erreur
     */
    public static function loadFromFile(string $filePath): ?array
    {
        if (!file_exists($filePath)) {
            return null;
        }

        $content = file_get_contents($filePath);
        if ($content === false) {
            return null;
        }

        $patterns = json_decode($content, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            return null;
        }

        return $patterns;
    }

    /**
     * Sauvegarde les patterns dans un fichier JSON
     *
     * @param string $filePath Chemin de destination
     * @param array $patterns Patterns à sauvegarder
     * @return bool Succès de l'opération
     */
    public static function saveToFile(string $filePath, array $patterns): bool
    {
        $json = json_encode($patterns, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
        if ($json === false) {
            return false;
        }

        return file_put_contents($filePath, $json) !== false;
    }
}
